import argparse
import openpyxl
import re
import os
from subprocess import Popen, DEVNULL
from wand.image import Image
from wand.color import Color

# XLSX columns
COL_FORMULA_A = 3
COL_FORMULA_B = 4
COL_NO = 5
COL_FILENAME_A = 6
COL_FILENAME_B = 7
COL_LATEX = 8

# Image base size = 1984 x 1299 - 9,1 x 5,9 cm
# Final size : 8,1 x 2,8 => 1766 * 616   85%, 30%
BB_WIDTH = 2800
BB_HEIGHT = 644

# Threshold for resizing
MAX_RESIZE = 1.1

# Output prefix
OUT_FOLDER = "nandeck-Tex\\"

parser = argparse.ArgumentParser(description='Produit les images pour Nandeck')
parser.add_argument("xlsFile", help="Fichier XLSX")

args = parser.parse_args()

wbInput = openpyxl.load_workbook(args.xlsFile)
wsInput = wbInput.active

# Dimensions
maxRow = wsInput.max_row
maxCol = wsInput.max_column
# print ("Rows: " + str(maxRow))
# print ("Cols: " + str(maxCol))


maxW = 0
maxH = 0

# Start at 2 to skip the header
for r in range(2,maxRow + 1):
        if wsInput.cell(row=r, column=COL_LATEX).value is not None:
                idx = wsInput.cell(row=r, column=COL_NO).value
                for alt in ['a', 'b']:
                        if alt == 'a':
                                formula = wsInput.cell(row=r, column=COL_FORMULA_A).value
                        else:
                                formula = wsInput.cell(row=r, column=COL_FORMULA_B).value

                        baseFN = "rem-" + str(idx) + "-" + alt

                        # Generate the LaTeX
                        
                        texFile = open(baseFN + ".tex", 'w')
                        texFile.write("\\documentclass[border=1pt,varwidth]{standalone}\n")
                        #texFile.write("\\usepackage{etex}\n")
                        #texFile.write("\\usepackage{m-pictex,m-ch-en}\n")
                        #texFile.write("\\usepackage{chemfig}\n")
                        #texFile.write("\\usepackage{pst-circ}\n")
                        #texFile.write("\\usepackage{pst-osci}\n")
                        #texFile.write("\\usepackage{pst-eucl}\n")
                        texFile.write("\\usepackage{pst-optic}\n")
                        texFile.write("\\usepackage[T1]{fontenc}\n")
                        texFile.write("\\usepackage[utf8]{inputenc}\n")
                        texFile.write("\\usepackage{amsmath,amssymb,amsfonts,amsthm}\n")
                        texFile.write("\\everymath{\displaystyle}\n")
                        texFile.write("\\begin{document}\n")
                        texFile.write("\\begin{minipage}{7.7cm}\n")
                        texFile.write("\\begin{center}\n")
                        texFile.write( str(formula)+"\n")
                        texFile.write("\\end{center}\n")
                        texFile.write("\\end{minipage}\n")
                        texFile.write("\\end{document}\n")
                        texFile.close()

                        # TODO : mettre tout ça dans un dossier temporaire
                        process = Popen(['pdflatex', '-interaction', 'batchmode', '-file-line-error', baseFN + ".tex"], env=dict(os.environ, max_print_line="1200"), stdout=DEVNULL)
                        process.communicate()

                        with Image(filename=baseFN + '.pdf', resolution=900) as img:

                                (w, h) = img.size

                                print (baseFN + " - " + str(w) + " x " + str(h))

                                if w > maxW:
                                        maxW = w
                                if h > maxH:
                                        maxH = h

                                wR = w/BB_WIDTH
                                hR = h/BB_HEIGHT

                                # Resizing
                                if wR > hR and wR > 1:
                                        if wR < MAX_RESIZE:
                                                img.resize(int(w / wR), int(h / wR))
                                                (w, h) = img.size
                                                print ("    Redimensionnée - " + str(w) + " x " + str(h))
                                        else:
                                                print ("    Image trop grande")
                                                continue
                                if hR > wR and hR > 1:
                                        if hR < MAX_RESIZE:
                                                img.resize(int(w / hR), int(h / hR))
                                                print ("    Redimensionnée - " + str(w) + " x " + str(h))
                                        else:
                                                print ("    Image trop grande")
                                                continue

                                with Image(width=BB_WIDTH, height=BB_HEIGHT, background=Color('white')) as out:
                                        lMargin = int((BB_WIDTH - w)/2)
                                        tMargin = int((BB_HEIGHT - h)/2)
                                        out.composite(img, left=lMargin, top=tMargin)
                                        out.format = 'png'
                                        out.save(filename=baseFN + ".png")
                                        if alt == 'a':
                                                wsInput.cell(row=r, column=COL_FILENAME_A).value = OUT_FOLDER + baseFN + ".png"
                                        else:
                                                wsInput.cell(row=r, column=COL_FILENAME_B).value = OUT_FOLDER + baseFN + ".png"

                                # Cleanup
                                os.remove(baseFN + ".tex")
                                os.remove(baseFN + ".aux")
                                os.remove(baseFN + ".log")
                                os.remove(baseFN + ".pdf")

wbInput.save(filename = args.xlsFile)
wbInput.close()
